# -*- coding: utf-8 -*-
"""
File: 
    tnames.py

Desrciption:  
    Finds all pytplot names that follow a pattern that may contain a wildcard.

Returns:
    list of str 
    A list of tplot names.
    
Parameters:
    pattern: str/list of str
        List of strings with names and/or patterns.
        If not given, it returns all pytplot names.

Notes:
    Allowed wildcards are ? for a single character, * from multiple characters.
    
"""

import pytplot
import fnmatch

def tnames(pattern=None):
    
    name_list = list()
    all_names = pytplot.tplot_names()
    
    if pattern is None:
        name_list.extend(all_names)
    else:
        if isinstance(pattern, str):
            name_list.extend(fnmatch.filter(all_names, pattern))
        else:
            for p in pattern: 
                name_list.extend(fnmatch.filter(all_names, p))
    
    return name_list 


# -*- coding: utf-8 -*-
"""
File: 
    ex_gmag.py

Desrciption:  
    Basic example with THEMIS GMAG data.
    Downloads THEMIS data from EPO GMAG stations and plots it.

"""

import pyspedas
import pytplot  

def ex_gmag():
    # Print the installed version of pyspedas
    pyspedas.get_spedas_prefs() 
    # Delete any existing pytplot variables
    pytplot.del_data()     
    # Define list of dates
    time_list = ['2015-12-31']
    # Get a list of EPO gmag stations
    sites = pyspedas.gmag_list(group='epo')
    #Download gmag files and load into pyspedas variables
    pyspedas.load_data('gmag', time_list, sites, '', '')
    #Get a list of loaded sites
    sites_loaded = pytplot.tplot_names()    
    #Subtact mean values 
    pyspedas.subtract_average(sites_loaded, '')
    #Download AE index data
    pyspedas.load_data('gmag', time_list, ['idx'], '', '')    
    #Plot 
    sites_loaded = pytplot.tplot_names()
    pytplot.tplot_options('title', 'EPO GMAG 2015-12-31')    
    pytplot.tplot(sites_loaded)

#Run the example code
#ex_gmag()


# -*- coding: utf-8 -*-
"""
File: 
    ex_create.py

Desrciption:  
    Creates a new pytplot object and applies analysis functions.

"""

import pyspedas
import pytplot
import numpy

def ex_create():
    pytplot.del_data()  
    #Create a sin wave plot
    a = list(range(0,101))
    b = [2.0/100.0*numpy.pi*s for s in a]
    c = pyspedas.time_float('2017-01-01')
    x = list()
    y = list()
    for i in range(len(b)):
        x.append(c+60.0/(2*numpy.pi)*60.0*b[i])
        y.append(1000.0*numpy.sin(b[i]))
    
    #Store data    
    pytplot.store_data('sinx', data={'x':x, 'y':y})
    #Apply clip
    pyspedas.tclip('sinx', -800.0, 800.0)
    #Remove NaN
    pyspedas.tdeflag('sinx-clip')    
    #Interpolate
    pyspedas.tinterpol(['sinx-clip-deflag'],['sinx'], 'quadratic')
    #Plot
    pytplot.ylim('sinx', -1100.0, 1100.0)
    pytplot.ylim('sinx-clip', -1100.0, 1100.0)
    pytplot.ylim('sinx-clip-deflag', -1100.0, 1100.0)
    pytplot.ylim('sinx-clip-deflag-itrp', -1100.0, 1100.0)
    pytplot.tplot(['sinx','sinx-clip', 'sinx-clip-deflag', 'sinx-clip-deflag-itrp'])
	
#Run the example code
#ex_create()


# -*- coding: utf-8 -*-
"""
File: 
    tdeflag.py

Desrciption:  
    Removes NaNs and other flags.
    
Parameters:
    names: str/list of str  
        List of pytplot names.
    method: str
        Method to apply. Default is 'remove_nan. 
        Other options 'repeat' (repeat last good value).
    flag: float
        Value to be replaced. Default is NaN.
    new_names: str/list of str
        List of new_names for pytplot variables.
        If '', then pytplot variables are replaced.
        If not given, then a suffix is applied. 
    suffix: 
        A suffix to apply. Default is '-clip'.
        
Notes:
    Allowed wildcards are ? for a single character, * from multiple characters.   
    Similar to tdeflag in IDL SPEDAS.

"""
import pyspedas
import pytplot
import numpy

def tdeflag(names, method=None, flag=None, new_names=None, suffix=None):
    
    old_names = pyspedas.tnames(names)
    
    if len(old_names) < 1:
        print('tdeflag error: No pytplot names were provided.')
        return 
    
    if suffix == None:
        suffix = '-deflag'
        
    if flag == None:
        flag = float('nan')       
    
    if new_names == None:
        n_names = [s + suffix for s in old_names]
    elif new_names == '':
        n_names = old_names
    else:
        n_names = new_names
    
    if len(n_names) != len(old_names):
        n_names = [s + suffix for s in old_names]
    
    for i in range(len(old_names)):
        time, data = pytplot.get_data(old_names[i])       
        new_time = []
        new_data = []
        for j in range(len(time)):
            if not numpy.isnan(data[j]):
                new_time.append(time[j])
                new_data.append(data[j])
        pytplot.store_data(n_names[i], data={'x':new_time, 'y':new_data})
        print('tdeflag was applied to: '+ n_names[i])


# -*- coding: utf-8 -*-
"""
File: 
    tclip.py

Desrciption:  
    Clip data and replace values with flag.
    
Parameters:
    names: str/list of str  
        List of pytplot names.
    ymin:
        Minimum value to clip
    ymax:
        Maximum value to clip
    flag:
        Values outside (ymin, ymax) are replaced with flag. Default is float('nan').
    new_names: str/list of str
        List of new_names for pytplot variables.
        If '', then pytplot variables are replaced.
        If not given, then a suffix is applied. 
    suffix: 
        A suffix to apply. Default is '-clip'.
        
Notes:
    Allowed wildcards are ? for a single character, * from multiple characters.   
    Similar to tclip in IDL SPEDAS.
    This function clips y-axis data. To clip time-axis, use time_clip.

"""
import pyspedas
import pytplot
import numpy

def tclip(names, ymin, ymax, flag=None, new_names=None, suffix=None):
    
    old_names = pyspedas.tnames(names)
    
    if len(old_names) < 1:
        print('tclip error: No pytplot names were provided.')
        return 
    
    if suffix == None:
        suffix = '-clip'
        
    if flag == None:
        flag = float('nan')       
    
    if new_names == None:
        n_names = [s + suffix for s in old_names]
    elif new_names == '':
        n_names = old_names
    else:
        n_names = new_names
    
    if len(n_names) != len(old_names):
        n_names = [s + suffix for s in old_names]
    
    for i in range(len(old_names)):
        time, data = pytplot.get_data(old_names[i])       
        new_data = numpy.array(data)
        new_data[new_data <= ymin] = flag 
        new_data[new_data >= ymax] = flag   
        pytplot.store_data(n_names[i], data={'x':time, 'y':new_data})
        print('tclip was applied to: '+ n_names[i])


# -*- coding: utf-8 -*-
"""
File: 
    ex_omni.py

Desrciption:  
    Basic example of plotting OMNI data.
    
"""

import pyspedas
import pytplot

def ex_omni():
    # Print the installed version of pyspedas
    pyspedas.get_spedas_prefs() 
    # Delete any existing pytplot variables
    pytplot.del_data()     
    # Download OMNI data for 2015-12-31
    pyspedas.load_data('omni', '2015-12-31', '', '', '1min')
    # OMNI loader downloads a full month of data, apply time_clip for a single day
    pyspedas.time_clip('flow_speed', '2015-12-31 00:00:00', '2015-12-31 23:59:59')
    # Plot
    pytplot.tplot_options('title', 'OMNI flow_speed 2015-12-31') 
    pytplot.tplot(['flow_speed-tclip'])

#Run the example code
#ex_omni()


# -*- coding: utf-8 -*-
"""
File: 
    tcopy.py

Desrciption:  
    Creates a deep copy of a tplot_variable, with a new name.
    
Parameters:
    names_in: str/list of str  
        List of pytplot names.
    names_out: str/list of str  
        List of pytplot names. If it is not provided, then suffix '-copy' is used. 
    suffix: 
        A suffix to apply. Default is '-copy'.
        
Notes:
    Allowed wildcards are ? for a single character, * from multiple characters.   

"""

import pytplot
import pyspedas
import copy

def tcopy_one(name_in, name_out):
    # Copies one pytplot variable
    tvar_old = pytplot.data_quants[name_in]
    tvar_new = copy.deepcopy(tvar_old)
    tvar_new.name = name_out
    pytplot.data_quants.update({name_out:tvar_new})
    print(name_in + ' copied to ' + name_out)
    
def tcopy(names_in, names_out=None, suffix=None):

    names_in = pyspedas.tnames(names_in)
    if len(names_in) < 1:
        print('tcopy error: No pytplot variables found.')
        return
    
    if suffix == None:
        suffix = '-copy' 
        
    if names_out is None:
        names_out = [s+suffix for s in names_in]    

    if isinstance(names_out, str):
        names_out = [names_out]  
    
    if len(names_in) != len(names_out):
        print('tcopy error: List with the names_in does not match list with the names out.')
        return
    
    for i in range(len(names_in)):
        n = names_in[i]
        o = names_out[i]
        if len(pyspedas.tnames(n)) == 1:
            tcopy_one(n, o)
        else:
            print('tplot name not found: ' + n)

        


# -*- coding: utf-8 -*-
"""
File: 
    tinterpol.py

Desrciption:  
    Interpolates data.
    
Parameters:
    names: str/list of str  
        List of pytplot names.
    interp_names: str/list of str  
        List of pytplot names. This should contain the new times array.
    method: str
        Interpolation method. Default is ‘linear’. 
        Specifies the kind of interpolation as a string (‘linear’, ‘nearest’, ‘zero’, ‘slinear’, 
        ‘quadratic’, ‘cubic’, ‘previous’, ‘next’, where ‘zero’, ‘slinear’, ‘quadratic’ and 
        ‘cubic’ refer to a spline interpolation of zeroth, first, second or third order; 
        ‘previous’ and ‘next’ simply return the previous or next value of the point) or 
        as an integer specifying the order of the spline interpolator to use. 
    new_names: str/list of str
        List of new_names for pytplot variables.
        If '', then pytplot variables are replaced.
        If not given, then a suffix is applied. 
    suffix: 
        A suffix to apply. Default is '-itrp'.
        
Notes:
    Allowed wildcards are ? for a single character, * from multiple characters.   
    Similar to tinterpol in IDL SPEDAS. 

"""
import pyspedas
import pytplot
import numpy
from scipy.interpolate import interp1d

def tinterpol(names, interp_names=None, method=None, new_names=None, suffix=None):
    
    old_names = pyspedas.tnames(names)
    
    if len(old_names) < 1:
        print('tinterpol error: No pytplot names were provided.')
        return 
    
    if suffix == None:
        suffix = '-itrp'   
    
    if method == None:
        method = 'linear'  
        
    if new_names == None:
        n_names = [s + suffix for s in old_names]
    elif new_names == '':
        n_names = old_names
    else:
        n_names = new_names
    
    if len(n_names) != len(old_names):
        n_names = [s + suffix for s in old_names]
    
    for i in range(len(old_names)):
        time, data = pytplot.get_data(old_names[i])   
        new_time, data1 = pytplot.get_data(interp_names[i])
        data = numpy.asarray(data).squeeze()
        f2 = interp1d(time, data, kind=method)
        new_data = f2(new_time)
        pytplot.store_data(n_names[i], data={'x':new_time, 'y':new_data})
        print('tinterpol (' + method + ') was applied to: '+ n_names[i])


# -*- coding: utf-8 -*-
"""
File: 
    subtract_median.py

Desrciption:  
    Time clip of data.
    
Parameters:
    names: str/list of str  
        List of pytplot names.
    new_names: str/list of str
        List of new_names for pytplot variables.
        If '', then pytplot variables are replaced.
        If not given, then a suffix is applied. 
    suffix: 
        A suffix to apply. Default is '-m'.
        
Notes:
    Allowed wildcards are ? for a single character, * from multiple characters.    

"""
import pyspedas
import pytplot

def time_clip(names, time_start, time_end, new_names=None, suffix=None):
    
    old_names = pyspedas.tnames(names)
    
    if len(old_names) < 1:
        print('Subtract Median error: No pytplot names were provided.')
        return 
    
    if suffix is None:
        suffix = '-tclip'
    
    if new_names is None:
        n_names = [s + suffix for s in old_names]
    elif new_names == '':
        n_names = old_names
    else:
        n_names = new_names
    
    if len(n_names) != len(old_names):
        n_names = [s + suffix for s in old_names]
    
    for j in range(len(old_names)):
        time, data = pytplot.get_data(old_names[j])  
        index_start = 0
        index_end = len(time)
        
        if index_end<1:
            print('Time clip found empty list.')
            continue
        
        new_time = pyspedas.time_float(time)
        new_time_start = pyspedas.time_float(time_start)
        new_time_end = pyspedas.time_float(time_end)
        
        if new_time_start>new_time_end:
            print('Time clip cannot work with start time larger than end time.')            
            continue
        
        if (new_time_start>new_time[-1]) or (new_time_end<new_time[0]):
            print('Time clip returns empty data.')            
            continue
        
        if (new_time_start<=new_time[0]) and (new_time_end>=new_time[-1]):
            print('Time clip returns full data set.')            
            continue            

        for i in range(index_end):
            if new_time[i]>=new_time_start:
                 index_start = i
                 break
        found_end = index_end     
        for i in range(index_start, index_end):
            if new_time[i]>new_time_end:
                found_end = i 
                break
        index_end = found_end          
        
        pytplot.store_data(n_names[j], data={'x':time[index_start:index_end], 'y':data[index_start:index_end, :]})
        print('Time clip was applied to: '+ n_names[j])
        